package main

import (
	"database/sql"
	"log"
	"log/slog"
	"net/http"
	"os"

	"gitea.michaelthomson.dev/mthomson/habits/internal/middleware"
	"gitea.michaelthomson.dev/mthomson/habits/internal/migrate"
	todohandler "gitea.michaelthomson.dev/mthomson/habits/internal/todo/handler"
	todorepository "gitea.michaelthomson.dev/mthomson/habits/internal/todo/repository/postgres"
	todoservice "gitea.michaelthomson.dev/mthomson/habits/internal/todo/service"
	_ "github.com/mattn/go-sqlite3"
	_ "github.com/jackc/pgx/v5/stdlib"
)

func main() {
	// create logger
	httpLogger := slog.New(slog.NewTextHandler(os.Stdout, nil))

	// create middlewares
	loggingMiddleware := middleware.LoggingMiddleware(httpLogger)

	// create db pool
	postgresUrl := "postgres://todo:password@localhost:5432/todo"
	db, err := sql.Open("pgx", postgresUrl)
	if err != nil {
		log.Fatal("Failed to open db pool")
	}
	defer db.Close()

	// run migrations
	migrate.Migrate(db)

	// create repos
	todoRepository := todorepository.NewPostgresTodoRepository(db)

	// create services
	todoService := todoservice.NewTodoService(todoRepository)

	// create mux
	mux := http.NewServeMux()

	// register handlers
	mux.Handle("GET /todo/{id}", loggingMiddleware(todohandler.HandleTodoGet(todoService)))
	mux.Handle("POST /todo", loggingMiddleware(todohandler.HandleTodoCreate(todoService)))
	mux.Handle("DELETE /todo/{id}", loggingMiddleware(todohandler.HandleTodoDelete(todoService)))
	mux.Handle("PUT /todo/{id}", loggingMiddleware(todohandler.HandleTodoUpdate(todoService)))

	// create server
	server := &http.Server{
		Addr: ":8080",
		Handler: mux,
	}

	server.ListenAndServe()
}
