package handler

import (
	"encoding/json"
	"fmt"
	"net/http"

	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/service"
)

type CreateTodoRequest struct {
	Name string `json:"name"`
	Done bool   `json:"done"`
}

func TodoFromCreateTodoRequest(todo CreateTodoRequest) service.Todo {
	return service.Todo{Id: 0, Name: todo.Name, Done: todo.Done}
}

type CreateTodoResponse struct {
	Id   int64  `json:"id"`
	Name string `json:"name"`
	Done bool   `json:"done"`
}

func CreateTodoResponseFromTodo(todo service.Todo) CreateTodoResponse {
	return CreateTodoResponse{Id: todo.Id, Name: todo.Name, Done: todo.Done}
}

func HandleTodoCreate(todoService TodoCreater) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		createTodoRequest := CreateTodoRequest{}
		decoder := json.NewDecoder(r.Body)
		decoder.DisallowUnknownFields()
		err := decoder.Decode(&createTodoRequest)

		if err != nil {
			http.Error(w, "", http.StatusBadRequest)
			return
		}

		todo, err := todoService.CreateTodo(TodoFromCreateTodoRequest(createTodoRequest))

		if err != nil {
			if err == service.ErrNotFound {
				http.Error(w, "", http.StatusNotFound)
				return
			}

			http.Error(w, "", http.StatusInternalServerError)
			return
		}

		response := CreateTodoResponseFromTodo(todo)

		w.Header().Set("Content-Type", "application/json")
		w.Header().Set("Location", fmt.Sprintf("/todo/%d", response.Id))
		w.WriteHeader(http.StatusCreated)

		err = json.NewEncoder(w).Encode(response)

		if err != nil {
			http.Error(w, "", http.StatusInternalServerError)
			return
		}

	}
}
