package handler

import (
	"encoding/json"
	"net/http"
	"strconv"

	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/service"
)

type UpdateTodoRequest struct {
	Name string `json:"name"`
	Done bool   `json:"done"`
}

func TodoFromUpdateTodoRequest(todo UpdateTodoRequest, id int64) service.Todo {
	return service.Todo{Id: id, Name: todo.Name, Done: todo.Done}
}

func HandleTodoUpdate(todoService TodoUpdater) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		updateTodoRequest := UpdateTodoRequest{}
		decoder := json.NewDecoder(r.Body)
		decoder.DisallowUnknownFields()
		err := decoder.Decode(&updateTodoRequest)

		if err != nil {
			http.Error(w, "", http.StatusBadRequest)
			return
		}

		idString := r.PathValue("id")

		id, err := strconv.ParseInt(idString, 10, 64)

		if err != nil {
			http.Error(w, "", http.StatusBadRequest)
			return
		}

		err = todoService.UpdateTodo(TodoFromUpdateTodoRequest(updateTodoRequest, id))

		if err != nil {
			if err == service.ErrNotFound {
				http.Error(w, "", http.StatusNotFound)
				return
			}

			http.Error(w, "", http.StatusInternalServerError)
			return
		}

		w.Header().Set("Content-Type", "application/json")
		w.WriteHeader(http.StatusOK)
	}
}
