package postgres

import (
	"database/sql"

	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/repository"
)

type PostgresTodoRepository struct {
	db *sql.DB
}

func NewPostgresTodoRepository(db *sql.DB) *PostgresTodoRepository {
	return &PostgresTodoRepository{
		db: db,
	}
}

func (r *PostgresTodoRepository) GetById(id int64) (repository.TodoRow, error) {
	todo := repository.TodoRow{}

	err := r.db.QueryRow("SELECT * FROM todo WHERE id = $1;", id).Scan(&todo.Id, &todo.Name, &todo.Done)

	if err != nil {
		if err == sql.ErrNoRows {
			return todo, repository.ErrNotFound
		}

		return todo, err
	}

	return todo, nil
}

func (r *PostgresTodoRepository) Create(todo repository.TodoRow) (repository.TodoRow, error) {
	result := r.db.QueryRow("INSERT INTO todo (name, done) VALUES ($1, $2) RETURNING id;", todo.Name, todo.Done)
	err := result.Scan(&todo.Id)

	if err != nil {
		return repository.TodoRow{}, err
	}

	return todo, nil
}

func (r *PostgresTodoRepository) Update(todo repository.TodoRow) error {
	result, err := r.db.Exec("UPDATE todo SET name = $1, done = $2 WHERE id = $3;", todo.Name, todo.Done, todo.Id)

	if err != nil {
		return err
	}

	rowsAffected, err := result.RowsAffected()

	if err != nil {
		return err
	}

	if rowsAffected == 0 {
		return repository.ErrNotFound
	}

	return nil
}

func (r *PostgresTodoRepository) Delete(id int64) error {
	result, err := r.db.Exec("DELETE FROM todo WHERE id = $1;", id)

	if err != nil {
		return err
	}

	rowsAffected, err := result.RowsAffected()

	if err != nil {
		return err
	}

	if rowsAffected == 0 {
		return repository.ErrNotFound
	}

	return nil
}
