package service_test

import (
	"testing"

	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/repository"
	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/repository/inmemory"
	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/service"
)

func TestCreateTodo(t *testing.T) {
	todoRepository := inmemory.NewInMemoryTodoRepository()

	todoService := service.NewTodoService(&todoRepository)

	t.Run("Create todo", func(t *testing.T) {
		todo := service.NewTodo("clean dishes", false)

		_, err := todoService.CreateTodo(todo)

		AssertNoError(t, err)
	})
}

func TestGetTodo(t *testing.T) {
	todoRepository := inmemory.NewInMemoryTodoRepository()

	todoRepository.Db[1] = repository.TodoRow{Id: 1, Name: "clean dishes", Done: false}

	todoService := service.NewTodoService(&todoRepository)

	t.Run("Get exisiting todo", func(t *testing.T) {
		_, err := todoService.GetTodo(1)

		AssertNoError(t, err)
	})

	t.Run("Get non-existant todo", func(t *testing.T) {
		_, err := todoService.GetTodo(2)

		AssertErrors(t, err, service.ErrNotFound)
	})
}

func TestDeleteTodo(t *testing.T) {
	todoRepository := inmemory.NewInMemoryTodoRepository()

	todoRepository.Db[1] = repository.TodoRow{Id: 1, Name: "clean dishes", Done: false}

	todoService := service.NewTodoService(&todoRepository)

	t.Run("Delete exisiting todo", func(t *testing.T) {
		err := todoService.DeleteTodo(1)

		AssertNoError(t, err)
	})

	t.Run("Delete non-existant todo", func(t *testing.T) {
		err := todoService.DeleteTodo(1)

		AssertErrors(t, err, service.ErrNotFound)
	})
}

func TestUpdateTodo(t *testing.T) {
	todoRepository := inmemory.NewInMemoryTodoRepository()

	todoRepository.Db[1] = repository.TodoRow{Id: 1, Name: "clean dishes", Done: false}

	todoService := service.NewTodoService(&todoRepository)

	t.Run("Update exisiting todo", func(t *testing.T) {
		todo := service.Todo{1, "clean dishes", true}

		err := todoService.UpdateTodo(todo)

		AssertNoError(t, err)

		newTodo, err := todoService.GetTodo(1)

		AssertNoError(t, err)

		AssertTodos(t, newTodo, todo)
	})

	t.Run("Update non-existant todo", func(t *testing.T) {
		todo := service.Todo{2, "clean dishes", true}

		err := todoService.UpdateTodo(todo)

		AssertErrors(t, err, service.ErrNotFound)
	})
}

func AssertErrors(t testing.TB, got, want error) {
	t.Helper()
	if got != want {
		t.Errorf("got error: %v, want error: %v", want, got)
	}
}

func AssertNoError(t testing.TB, err error) {
	t.Helper()
	if err != nil {
		t.Errorf("expected no error, got %v", err)
	}
}

func AssertTodos(t testing.TB, got, want service.Todo) {
	if !got.Equal(want) {
		t.Errorf("got %+v want %+v", got, want)
	}
}
